@@ -1,5 +1,5 @@
 /*  GNU ed - The GNU line editor.
-    Copyright (C) 2006-2017 Antonio Diaz Diaz.
+    Copyright (C) 2006-2020 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,10 +42,9 @@
 #include "ed.h"
 
 
-static const char * const Program_name = "GNU Ed";
 static const char * const program_name = "ed";
-static const char * const program_year = "2017";
-static const char * invocation_name = 0;
+static const char * const program_year = "2020";
+static const char * invocation_name = "ed";		/* default value */
 
 static bool restricted_ = false;	/* if set, run in restricted mode */
 static bool scripted_ = false;		/* if set, suppress diagnostics,
@@ -60,8 +59,14 @@ bool traditional( void ) { return traditional_; }
 
 static void show_help( void )
   {
-  printf( "%s - The GNU line editor.\n", Program_name );
-  printf( "\nUsage: %s [options] [file]\n", invocation_name );
+  printf( "GNU ed is a line-oriented text editor. It is used to create, display,\n"
+          "modify and otherwise manipulate text files, both interactively and via\n"
+          "shell scripts. A restricted version of ed, red, can only edit files in\n"
+          "the current directory and cannot execute shell commands. Ed is the\n"
+          "'standard' text editor in the sense that it is the original editor for\n"
+          "Unix, and thus widely available. For most purposes, however, it is\n"
+          "superseded by full-screen editors such as GNU Emacs or GNU Moe.\n"
+          "\nUsage: %s [options] [file]\n", invocation_name );
   printf( "\nOptions:\n"
           "  -h, --help                 display this help and exit\n"
           "  -V, --version              output version information and exit\n"
@@ -71,7 +76,7 @@ static void show_help( void )
           "  -r, --restricted           run in restricted mode\n"
           "  -s, --quiet, --silent      suppress diagnostics, byte counts and '!' prompt\n"
           "  -v, --verbose              be verbose; equivalent to the 'H' command\n"
-          "Start edit by reading in 'file' if given.\n"
+          "\nStart edit by reading in 'file' if given.\n"
           "If 'file' begins with a '!', read output of shell command.\n"
           "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
           "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
@@ -107,11 +112,9 @@ void show_strerror( const char * const filename, const int errcode )
 static void show_error( const char * const msg, const int errcode, const bool help )
   {
   if( msg && msg[0] )
-    {
-    fprintf( stderr, "%s: %s", program_name, msg );
-    if( errcode > 0 ) fprintf( stderr, ": %s", strerror( errcode ) );
-    fputc( '\n', stderr );
-    }
+    fprintf( stderr, "%s: %s%s%s\n", program_name, msg,
+             ( errcode > 0 ) ? ": " : "",
+             ( errcode > 0 ) ? strerror( errcode ) : "" );
   if( help )
     fprintf( stderr, "Try '%s --help' for more information.\n",
              invocation_name );
@@ -156,7 +159,7 @@ int main( const int argc, const char * const argv[] )
     {  0 ,  0,                  ap_no } };
 
   struct Arg_parser parser;
-  invocation_name = argv[0];
+  if( argc > 0 ) invocation_name = argv[0];
 
   if( !ap_init( &parser, argc, argv, options, 0 ) )
     { show_error( "Memory exhausted.", 0, false ); return 1; }
@@ -173,7 +176,7 @@ int main( const int argc, const char * const argv[] )
       case 'G': traditional_ = true; break;	/* backward compatibility */
       case 'h': show_help(); return 0;
       case 'l': loose = true; break;
-      case 'p': set_prompt( arg ); break;
+      case 'p': if( set_prompt( arg ) ) break; else return 1;
       case 'r': restricted_ = true; break;
       case 's': scripted_ = true; break;
       case 'v': set_verbose(); break;
@@ -194,7 +197,7 @@ int main( const int argc, const char * const argv[] )
       {
       if( read_file( arg, 0 ) < 0 && is_regular_file( 0 ) )
         return 2;
-      else if( arg[0] != '!' ) set_def_filename( arg );
+      else if( arg[0] != '!' && !set_def_filename( arg ) ) return 1;
       }
     else
       {
